#ifndef AVAILABILITY_H
#define AVAILABILITY_H

#include <gsl/gsl_rng.h>

typedef enum {
  HOST_UP=0,
  HOST_RECLAIMED=1,
  HOST_DOWN=2
} hoststate_t;

typedef enum {
  MARKOV=0,
  PARETO=1,
  WEIBULL=2,
} distribution_t;

typedef struct _markov_chain_t markov_chain_t;
typedef struct _availability_trace_t availability_trace_t;

/* Data structure for a markov chain */
struct _markov_chain_t {
  double matrix[3][3];	/* transition matrix */
  double pi[3];		/* pi distribution */
};

/* Datat structure for an availability trace */
struct _availability_trace_t {
  int num_entries;
  int start_entry; 	/* Entry point to the trace */
  int current_entry;    /* Useful to keep track of where we are */
  long int time_offset; /* To implement wrap-around availability */
  long int *dates;
  char *states; 
};

/* Prototypes */

int computePiDistribution(markov_chain_t *chain);
int pickAmongThree(double *p, gsl_rng *rng);

int buildChainFromTrace(markov_chain_t *chain, availability_trace_t *trace);
int buildTraceFromChain(int seed, long int length, distribution_t dist, 
double dist_shape, markov_chain_t *chain, availability_trace_t *trace);
int parseTraceFile(char *file, availability_trace_t *trace);
void printTrace(availability_trace_t *trace);
void printChain(markov_chain_t *chain);

#endif
