#include <stdio.h>
#include <math.h>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>


#include "markov.h"

#define EPSILON 0.00000001

/* computePiDistribution(): for a 3x3 chain */
int computePiDistribution(markov_chain *chain)
{

  double uu, ur, ud;
  double ru, rr, rd;
  double du, dr, dd;

  double *pu = &(chain->pi[0]);
  double *pr = &(chain->pi[1]);
  double *pd = &(chain->pi[2]);

  uu = chain->matrix[0][0];
  ur = chain->matrix[0][1];
  ud = chain->matrix[0][2];
  ru = chain->matrix[1][0];
  rr = chain->matrix[1][1];
  rd = chain->matrix[1][2];
  du = chain->matrix[2][0];
  dr = chain->matrix[2][1];
  dd = chain->matrix[2][2];

  if (fabs(ru - du) < EPSILON) {
    *pu = du / (1 - uu + du);
    *pr = (-dr -(ur - dr)*(*pu)) / (rr -1 - dr);
    *pd = 1 - *pu - *pr;
  } else if (fabs(ur - dr) < EPSILON) {
    *pr = dr / (1 - rr + dr);
    *pu = (-du -(ru - du)*(*pr)) / (uu -1 - du);
    *pd = 1 - *pu - *pr;
  } else {
    *pr = (-du + dr *(uu -1 -du)/(ur-dr)) / (ru - du - (rr - 1 -dr)*(uu -1 -du)/(ur - dr));
    *pu = (-dr - (rr -1 - dr) * (*pr)) / (ur - dr);
    *pd = 1 - *pr - *pu;
  }
  return 0;
}

/* pickAmongThree() */
int pickAmongThree(double *p, gsl_rng *rng)
{
  double ran;

  if (fabs(p[0]+p[1]+p[2] - 1) > EPSILON) {
    fprintf(stderr,"pickAmongThree(): probabilities don't add to 1\n");
    return -1;
  }

  ran=gsl_rng_uniform(rng);

  if (ran <= p[0]) {
    return 0;
  } else if (ran <= p[0]+p[1]) {
    return 1;
  } else {
    return 2;
  }
}


