#ifndef SIMULATOR_H
#define SIMULATOR_H

#include "availability.h"

/* Convenient Macros */

#ifndef MIN
#define MIN(x,y) ((x) > (y) ? (y) : (x))
#endif 

#ifndef MAX   
#define MAX(x,y) ((x) < (y) ? (y) : (x))
#endif

#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif

#define STATE(host) ((host)->availability.states[(host)->availability.current_entry])

#define PRINT_VERBOSE0(string) {if (global.verbose>=1) {fprintf(stderr,(string));}}
#define PRINT_VERBOSE1(string,arg1) {if (global.verbose>=1) {fprintf(stderr,(string),(arg1));}}
#define PRINT_VERBOSE2(string,arg1,arg2) {if (global.verbose>=1) {fprintf(stderr,(string),(arg1),(arg2));}}
#define PRINT_VERBOSE3(string,arg1,arg2,arg3) {if (global.verbose>=1) {fprintf(stderr,(string),(arg1),(arg2),(arg3));}}
#define PRINT_VERBOSE4(string,arg1,arg2,arg3,arg4) {if (global.verbose>=1) {fprintf(stderr,(string),(arg1),(arg2),(arg3),(arg4));}}
#define PRINT_VERBOSE5(string,arg1,arg2,arg3,arg4,arg5) {if (global.verbose>=1) {fprintf(stderr,(string),(arg1),(arg2),(arg3),(arg4),(arg5));}}

#define PRINT_VVERBOSE0(string) {if (global.verbose>=2) {fprintf(stderr,(string));}}
#define PRINT_VVERBOSE1(string,arg1) {if (global.verbose>=2) { fprintf(stderr,(string),(arg1));}}
#define PRINT_VVERBOSE2(string,arg1,arg2) {if (global.verbose>=2) { fprintf(stderr,(string),(arg1),(arg2));}}
#define PRINT_VVERBOSE3(string,arg1,arg2,arg3) {if (global.verbose>=2) { fprintf(stderr,(string),(arg1),(arg2),(arg3));}}
#define PRINT_VVERBOSE4(string,arg1,arg2,arg3,arg4) {if (global.verbose>=2) { fprintf(stderr,(string),(arg1),(arg2),(arg3),(arg4));}}

#define EPSILON 0.00000001

#define MODE_INDEPENDENT 0
#define MODE_COUPLED     1

/* 
 * Data types
 */
typedef enum {
  ALG_RANDOM=0,
  ALG_INDEP_WEIGHTED_RANDOM_Puu,
  ALG_INDEP_WEIGHTED_RANDOM_Puuw,
  ALG_INDEP_WEIGHTED_RANDOM_Pfanny,
  ALG_INDEP_WEIGHTED_RANDOM_Pfannyw,
  ALG_INDEP_WEIGHTED_RANDOM_PIu,
  ALG_INDEP_WEIGHTED_RANDOM_PIuw,
  ALG_INDEP_WEIGHTED_RANDOM_PIuf,
  ALG_INDEP_WEIGHTED_RANDOM_PIufw,
  ALG_INDEP_MCT_v1,
  ALG_INDEP_MCT_v2,
  ALG_INDEP_LW_v1,
  ALG_INDEP_LW_v2,
  ALG_INDEP_UD_v1,
  ALG_INDEP_UD_v2,
  ALG_INDEP_RATE_v1,
  ALG_INDEP_RATE_v2,
  ALG_INDEP_EMCT_v1,
  ALG_INDEP_EMCT_v2,
  ALG_INDEP_ELW_v1,
  ALG_INDEP_ELW_v2,
  ALG_INDEP_EUD_v1,
  ALG_INDEP_EUD_v2,
  ALG_INDEP_ERATE_v1,
  ALG_INDEP_ERATE_v2,
  ALG_INDEP_ENCRATE_v1,
  ALG_INDEP_ENCRATE_v2,
  ALG_INDEP_ENCRATENAIVE_v1,
  ALG_INDEP_ENCRATENAIVE_v2,
  ALG_COUPLED_RANDOM,
  ALG_COUPLED_WEIGHTED_RANDOM_Puu,
  ALG_COUPLED_WEIGHTED_RANDOM_Puuw,
  ALG_COUPLED_WEIGHTED_RANDOM_Pfanny,
  ALG_COUPLED_WEIGHTED_RANDOM_Pfannyw,
  ALG_COUPLED_WEIGHTED_RANDOM_PIu,
  ALG_COUPLED_WEIGHTED_RANDOM_PIuw,
  ALG_COUPLED_WEIGHTED_RANDOM_PIuf,
  ALG_COUPLED_WEIGHTED_RANDOM_PIufw,
  ALG_COUPLED_MCT_v1,
  ALG_COUPLED_MCT_v2,
  ALG_COUPLED_LW_v1,
  ALG_COUPLED_LW_v2,
  ALG_COUPLED_UD_v1,
  ALG_COUPLED_UD_v2,
  ALG_COUPLED_EMCT_v1,
  ALG_COUPLED_EMCT_v2,
  ALG_COUPLED_ELW_v1,
  ALG_COUPLED_ELW_v2,
  ALG_COUPLED_EUD_v1,
  ALG_COUPLED_EUD_v2,
  ALG_COUPLED_IE,
  ALG_COUPLED_IP,
  ALG_COUPLED_IR,
  ALG_COUPLED_IY,
  ALG_COUPLED_RANDOM_PP,
  ALG_COUPLED_WEIGHTED_RANDOM_Puu_PP,
  ALG_COUPLED_WEIGHTED_RANDOM_Puuw_PP,
  ALG_COUPLED_WEIGHTED_RANDOM_Pfanny_PP,
  ALG_COUPLED_WEIGHTED_RANDOM_Pfannyw_PP,
  ALG_COUPLED_WEIGHTED_RANDOM_PIu_PP,
  ALG_COUPLED_WEIGHTED_RANDOM_PIuw_PP,
  ALG_COUPLED_WEIGHTED_RANDOM_PIuf_PP,
  ALG_COUPLED_WEIGHTED_RANDOM_PIufw_PP,
  ALG_COUPLED_MCT_v1_PP,
  ALG_COUPLED_MCT_v2_PP,
  ALG_COUPLED_LW_v1_PP,
  ALG_COUPLED_LW_v2_PP,
  ALG_COUPLED_UD_v1_PP,
  ALG_COUPLED_UD_v2_PP,
  ALG_COUPLED_EMCT_v1_PP,
  ALG_COUPLED_EMCT_v2_PP,
  ALG_COUPLED_ELW_v1_PP,
  ALG_COUPLED_ELW_v2_PP,
  ALG_COUPLED_EUD_v1_PP,
  ALG_COUPLED_EUD_v2_PP,
  ALG_COUPLED_IE_PP,
  ALG_COUPLED_IP_PP,
  ALG_COUPLED_IR_PP,
  ALG_COUPLED_IY_PP,
  ALG_COUPLED_RANDOM_PE,
  ALG_COUPLED_WEIGHTED_RANDOM_Puu_PE,
  ALG_COUPLED_WEIGHTED_RANDOM_Puuw_PE,
  ALG_COUPLED_WEIGHTED_RANDOM_Pfanny_PE,
  ALG_COUPLED_WEIGHTED_RANDOM_Pfannyw_PE,
  ALG_COUPLED_WEIGHTED_RANDOM_PIu_PE,
  ALG_COUPLED_WEIGHTED_RANDOM_PIuw_PE,
  ALG_COUPLED_WEIGHTED_RANDOM_PIuf_PE,
  ALG_COUPLED_WEIGHTED_RANDOM_PIufw_PE,
  ALG_COUPLED_MCT_v1_PE,
  ALG_COUPLED_MCT_v2_PE,
  ALG_COUPLED_LW_v1_PE,
  ALG_COUPLED_LW_v2_PE,
  ALG_COUPLED_UD_v1_PE,
  ALG_COUPLED_UD_v2_PE,
  ALG_COUPLED_EMCT_v1_PE,
  ALG_COUPLED_EMCT_v2_PE,
  ALG_COUPLED_ELW_v1_PE,
  ALG_COUPLED_ELW_v2_PE,
  ALG_COUPLED_EUD_v1_PE,
  ALG_COUPLED_EUD_v2_PE,
  ALG_COUPLED_IE_PE,
  ALG_COUPLED_IP_PE,
  ALG_COUPLED_IR_PE,
  ALG_COUPLED_IY_PE,
  ALG_COUPLED_RANDOM_PR,
  ALG_COUPLED_WEIGHTED_RANDOM_Puu_PR,
  ALG_COUPLED_WEIGHTED_RANDOM_Puuw_PR,
  ALG_COUPLED_WEIGHTED_RANDOM_Pfanny_PR,
  ALG_COUPLED_WEIGHTED_RANDOM_Pfannyw_PR,
  ALG_COUPLED_WEIGHTED_RANDOM_PIu_PR,
  ALG_COUPLED_WEIGHTED_RANDOM_PIuw_PR,
  ALG_COUPLED_WEIGHTED_RANDOM_PIuf_PR,
  ALG_COUPLED_WEIGHTED_RANDOM_PIufw_PR,
  ALG_COUPLED_MCT_v1_PR,
  ALG_COUPLED_MCT_v2_PR,
  ALG_COUPLED_LW_v1_PR,
  ALG_COUPLED_LW_v2_PR,
  ALG_COUPLED_UD_v1_PR,
  ALG_COUPLED_UD_v2_PR,
  ALG_COUPLED_EMCT_v1_PR,
  ALG_COUPLED_EMCT_v2_PR,
  ALG_COUPLED_ELW_v1_PR,
  ALG_COUPLED_ELW_v2_PR,
  ALG_COUPLED_EUD_v1_PR,
  ALG_COUPLED_EUD_v2_PR,
  ALG_COUPLED_IE_PR,
  ALG_COUPLED_IP_PR,
  ALG_COUPLED_IR_PR,
  ALG_COUPLED_IY_PR,
  ALG_COUPLED_RANDOM_PY,
  ALG_COUPLED_WEIGHTED_RANDOM_Puu_PY,
  ALG_COUPLED_WEIGHTED_RANDOM_Puuw_PY,
  ALG_COUPLED_WEIGHTED_RANDOM_Pfanny_PY,
  ALG_COUPLED_WEIGHTED_RANDOM_Pfannyw_PY,
  ALG_COUPLED_WEIGHTED_RANDOM_PIu_PY,
  ALG_COUPLED_WEIGHTED_RANDOM_PIuw_PY,
  ALG_COUPLED_WEIGHTED_RANDOM_PIuf_PY,
  ALG_COUPLED_WEIGHTED_RANDOM_PIufw_PY,
  ALG_COUPLED_MCT_v1_PY,
  ALG_COUPLED_MCT_v2_PY,
  ALG_COUPLED_LW_v1_PY,
  ALG_COUPLED_LW_v2_PY,
  ALG_COUPLED_UD_v1_PY,
  ALG_COUPLED_UD_v2_PY,
  ALG_COUPLED_EMCT_v1_PY,
  ALG_COUPLED_EMCT_v2_PY,
  ALG_COUPLED_ELW_v1_PY,
  ALG_COUPLED_ELW_v2_PY,
  ALG_COUPLED_EUD_v1_PY,
  ALG_COUPLED_EUD_v2_PY,
  ALG_COUPLED_IE_PY,
  ALG_COUPLED_IP_PY,
  ALG_COUPLED_IR_PY,
  ALG_COUPLED_IY_PY,
  ALG_NUM_ALGS
} algorithm_t;


typedef struct _task_t task_t;
typedef struct _host_t host_t;
typedef struct _application_t application_t;
typedef struct _platform_t platform_t;
typedef struct _config_t config_t;


struct _application_t {
  int mode;			/* MODE_COUPLED or MODE_INDEPENDENT	    */
  double *remaining;		/* Fraction remaining until completion      */
  int Tprog;			/* Time to receive application program      */
  int Tdata;			/* Time to receive application data         */
  int num_completed_iterations; /* number of completed iterations           */
  long date_of_current_iteration;/*date of begin of current iteration*/
  int max_num_iterations;	/* Max number of iterations                 */
  long int timelimit;           /* Max number of time steps                 */
  int num_tasks;		/* Number of tasks			    */
};

struct _platform_t {
  int num_hosts;             /* Number of hosts */
  int max_num_comms;         /* Max number of current pending comms */
  int nonresumabledownloads; /* TRUE or FALSE */
  host_t **hosts;
};

struct _task_t {
  int id;
  int activated;	          /* Clones are initially inactive */
  task_t *clone_of;	          /* NULL if original task */
  host_t *assigned_host;          /* NULL if not assigned to a host */
  int time_to_data_download;      /* if 0, data is there */
  double remaining_work_fraction; /* 1.0 if not started */
  int done;                       /* = 1 if the task is finished */
};

struct _host_t {
  int id;
  int seed;
  int num_assigned_tasks;
  int picking_order; /* The order in which the proc was picked when assigning tasks */
  int has_communicated_this_step; /* If the host has already done a comm this step */
  availability_trace_t availability;  /* Availablity trace */
  markov_chain_t chain;               /* Specified or Discovered */
  double Pfanny;
  double PUD_helper;
  double PLW_helper;
  int w;
  int data_download_time;
  int time_to_prog_download;          /* if 0, program is there */
};

struct _config_t {
  int num_tasks;
  task_t **tasks;
};

/*
 * Global variables 
 */

struct global_t {
  int verbose;			/* 0, 1, 2, ...   */
  int num_algs;

  distribution_t distribution;
  double distribution_shape;

  int max_num_clones;

  long current_date;	// Simulated date

  /* Scheduling algorithms to execute */
  algorithm_t *algs;		

  platform_t platform;
  application_t application;	

  /* Configuration: encapsulates tasks */
  config_t *config;

  /* Performance/Info counter */
  double available_compute_power;  // The work provided by the application
  double used_compute_power;       // The work used by the application 
  double wasted_compute_power;     // The work wasted by the applications
  long comm_timesteps;             // Total communication time-steps
  long comm_wasted_timesteps;      // Total communication wasted time-steps
};

typedef int (*schedule_function_t)(config_t *);
int schedule_indep_random(config_t *);
int schedule_indep_weighted_random_Puu(config_t *);
int schedule_indep_weighted_random_Puuw(config_t *);
int schedule_indep_weighted_random_Pfanny(config_t *);
int schedule_indep_weighted_random_Pfannyw(config_t *);
int schedule_indep_weighted_random_PIu(config_t *);
int schedule_indep_weighted_random_PIuw(config_t *);
int schedule_indep_weighted_random_PIuf(config_t *);
int schedule_indep_weighted_random_PIufw(config_t *);
int schedule_indep_MCT_v1(config_t *);
int schedule_indep_MCT_v2(config_t *);
int schedule_indep_LW_v1(config_t *);
int schedule_indep_LW_v2(config_t *);
int schedule_indep_UD_v1(config_t *);
int schedule_indep_UD_v2(config_t *);
int schedule_indep_RATE_v1(config_t *);
int schedule_indep_RATE_v2(config_t *);
int schedule_indep_EMCT_v1(config_t *);
int schedule_indep_EMCT_v2(config_t *);
int schedule_indep_ELW_v1(config_t *);
int schedule_indep_ELW_v2(config_t *);
int schedule_indep_EUD_v1(config_t *);
int schedule_indep_EUD_v2(config_t *);
int schedule_indep_ERATE_v1(config_t *);
int schedule_indep_ERATE_v2(config_t *);
int schedule_indep_ENCRATE_v1(config_t *);
int schedule_indep_ENCRATE_v2(config_t *);
int schedule_indep_ENCRATENAIVE_v1(config_t *);
int schedule_indep_ENCRATENAIVE_v2(config_t *);
int schedule_coupled_random(config_t *);
int schedule_coupled_weighted_random_Puu(config_t *);
int schedule_coupled_weighted_random_Puuw(config_t *);
int schedule_coupled_weighted_random_Pfanny(config_t *);
int schedule_coupled_weighted_random_Pfannyw(config_t *);
int schedule_coupled_weighted_random_PIu(config_t *);
int schedule_coupled_weighted_random_PIuw(config_t *);
int schedule_coupled_weighted_random_PIuf(config_t *);
int schedule_coupled_weighted_random_PIufw(config_t *);
int schedule_coupled_MCT_v1(config_t *);
int schedule_coupled_MCT_v2(config_t *);
int schedule_coupled_LW_v1(config_t *);
int schedule_coupled_LW_v2(config_t *);
int schedule_coupled_UD_v1(config_t *);
int schedule_coupled_UD_v2(config_t *);
int schedule_coupled_EMCT_v1(config_t *);
int schedule_coupled_EMCT_v2(config_t *);
int schedule_coupled_ELW_v1(config_t *);
int schedule_coupled_ELW_v2(config_t *);
int schedule_coupled_EUD_v1(config_t *);
int schedule_coupled_EUD_v2(config_t *);
int schedule_coupled_IE(config_t *);
int schedule_coupled_IP(config_t *);
int schedule_coupled_IR(config_t *);
int schedule_coupled_IY(config_t *);
int schedule_coupled_random_PP(config_t *);
int schedule_coupled_weighted_random_Puu_PP(config_t *);
int schedule_coupled_weighted_random_Puuw_PP(config_t *);
int schedule_coupled_weighted_random_Pfanny_PP(config_t *);
int schedule_coupled_weighted_random_Pfannyw_PP(config_t *);
int schedule_coupled_weighted_random_PIu_PP(config_t *);
int schedule_coupled_weighted_random_PIuw_PP(config_t *);
int schedule_coupled_weighted_random_PIuf_PP(config_t *);
int schedule_coupled_weighted_random_PIufw_PP(config_t *);
int schedule_coupled_MCT_v1_PP(config_t *);
int schedule_coupled_MCT_v2_PP(config_t *);
int schedule_coupled_LW_v1_PP(config_t *);
int schedule_coupled_LW_v2_PP(config_t *);
int schedule_coupled_UD_v1_PP(config_t *);
int schedule_coupled_UD_v2_PP(config_t *);
int schedule_coupled_EMCT_v1_PP(config_t *);
int schedule_coupled_EMCT_v2_PP(config_t *);
int schedule_coupled_ELW_v1_PP(config_t *);
int schedule_coupled_ELW_v2_PP(config_t *);
int schedule_coupled_EUD_v1_PP(config_t *);
int schedule_coupled_EUD_v2_PP(config_t *);
int schedule_coupled_IE_PP(config_t *);
int schedule_coupled_IP_PP(config_t *);
int schedule_coupled_IR_PP(config_t *);
int schedule_coupled_IY_PP(config_t *);
int schedule_coupled_random_PE(config_t *);
int schedule_coupled_weighted_random_Puu_PE(config_t *);
int schedule_coupled_weighted_random_Puuw_PE(config_t *);
int schedule_coupled_weighted_random_Pfanny_PE(config_t *);
int schedule_coupled_weighted_random_Pfannyw_PE(config_t *);
int schedule_coupled_weighted_random_PIu_PE(config_t *);
int schedule_coupled_weighted_random_PIuw_PE(config_t *);
int schedule_coupled_weighted_random_PIuf_PE(config_t *);
int schedule_coupled_weighted_random_PIufw_PE(config_t *);
int schedule_coupled_MCT_v1_PE(config_t *);
int schedule_coupled_MCT_v2_PE(config_t *);
int schedule_coupled_LW_v1_PE(config_t *);
int schedule_coupled_LW_v2_PE(config_t *);
int schedule_coupled_UD_v1_PE(config_t *);
int schedule_coupled_UD_v2_PE(config_t *);
int schedule_coupled_EMCT_v1_PE(config_t *);
int schedule_coupled_EMCT_v2_PE(config_t *);
int schedule_coupled_ELW_v1_PE(config_t *);
int schedule_coupled_ELW_v2_PE(config_t *);
int schedule_coupled_EUD_v1_PE(config_t *);
int schedule_coupled_EUD_v2_PE(config_t *);
int schedule_coupled_IE_PE(config_t *);
int schedule_coupled_IP_PE(config_t *);
int schedule_coupled_IR_PE(config_t *);
int schedule_coupled_IY_PE(config_t *);
int schedule_coupled_random_PR(config_t *);
int schedule_coupled_weighted_random_Puu_PR(config_t *);
int schedule_coupled_weighted_random_Puuw_PR(config_t *);
int schedule_coupled_weighted_random_Pfanny_PR(config_t *);
int schedule_coupled_weighted_random_Pfannyw_PR(config_t *);
int schedule_coupled_weighted_random_PIu_PR(config_t *);
int schedule_coupled_weighted_random_PIuw_PR(config_t *);
int schedule_coupled_weighted_random_PIuf_PR(config_t *);
int schedule_coupled_weighted_random_PIufw_PR(config_t *);
int schedule_coupled_MCT_v1_PR(config_t *);
int schedule_coupled_MCT_v2_PR(config_t *);
int schedule_coupled_LW_v1_PR(config_t *);
int schedule_coupled_LW_v2_PR(config_t *);
int schedule_coupled_UD_v1_PR(config_t *);
int schedule_coupled_UD_v2_PR(config_t *);
int schedule_coupled_EMCT_v1_PR(config_t *);
int schedule_coupled_EMCT_v2_PR(config_t *);
int schedule_coupled_ELW_v1_PR(config_t *);
int schedule_coupled_ELW_v2_PR(config_t *);
int schedule_coupled_EUD_v1_PR(config_t *);
int schedule_coupled_EUD_v2_PR(config_t *);
int schedule_coupled_IE_PR(config_t *);
int schedule_coupled_IP_PR(config_t *);
int schedule_coupled_IR_PR(config_t *);
int schedule_coupled_IY_PR(config_t *);
int schedule_coupled_random_PY(config_t *);
int schedule_coupled_weighted_random_Puu_PY(config_t *);
int schedule_coupled_weighted_random_Puuw_PY(config_t *);
int schedule_coupled_weighted_random_Pfanny_PY(config_t *);
int schedule_coupled_weighted_random_Pfannyw_PY(config_t *);
int schedule_coupled_weighted_random_PIu_PY(config_t *);
int schedule_coupled_weighted_random_PIuw_PY(config_t *);
int schedule_coupled_weighted_random_PIuf_PY(config_t *);
int schedule_coupled_weighted_random_PIufw_PY(config_t *);
int schedule_coupled_MCT_v1_PY(config_t *);
int schedule_coupled_MCT_v2_PY(config_t *);
int schedule_coupled_LW_v1_PY(config_t *);
int schedule_coupled_LW_v2_PY(config_t *);
int schedule_coupled_UD_v1_PY(config_t *);
int schedule_coupled_UD_v2_PY(config_t *);
int schedule_coupled_EMCT_v1_PY(config_t *);
int schedule_coupled_EMCT_v2_PY(config_t *);
int schedule_coupled_ELW_v1_PY(config_t *);
int schedule_coupled_ELW_v2_PY(config_t *);
int schedule_coupled_EUD_v1_PY(config_t *);
int schedule_coupled_EUD_v2_PY(config_t *);
int schedule_coupled_IE_PY(config_t *);
int schedule_coupled_IP_PY(config_t *);
int schedule_coupled_IR_PY(config_t *);
int schedule_coupled_IY_PY(config_t *);

extern const char *algorithm_names[ALG_NUM_ALGS];


static const schedule_function_t schedule_functions[ALG_NUM_ALGS]={
        	schedule_indep_random,
        	schedule_indep_weighted_random_Puu,
        	schedule_indep_weighted_random_Puuw,
        	schedule_indep_weighted_random_Pfanny,
        	schedule_indep_weighted_random_Pfannyw,
        	schedule_indep_weighted_random_PIu,
        	schedule_indep_weighted_random_PIuw,
        	schedule_indep_weighted_random_PIuf,
        	schedule_indep_weighted_random_PIufw,
        	schedule_indep_MCT_v1,
        	schedule_indep_MCT_v2,
        	schedule_indep_LW_v1,
        	schedule_indep_LW_v2,
        	schedule_indep_UD_v1,
        	schedule_indep_UD_v2,
        	schedule_indep_RATE_v1,
        	schedule_indep_RATE_v2,
        	schedule_indep_EMCT_v1,
        	schedule_indep_EMCT_v2,
        	schedule_indep_ELW_v1,
        	schedule_indep_ELW_v2,
        	schedule_indep_EUD_v1,
        	schedule_indep_EUD_v2,
        	schedule_indep_ERATE_v1,
        	schedule_indep_ERATE_v2,
        	schedule_indep_ENCRATE_v1,
        	schedule_indep_ENCRATE_v2,
        	schedule_indep_ENCRATENAIVE_v1,
        	schedule_indep_ENCRATENAIVE_v2,
        	schedule_coupled_random,
                schedule_coupled_weighted_random_Puu,
                schedule_coupled_weighted_random_Puuw,
                schedule_coupled_weighted_random_Pfanny,
                schedule_coupled_weighted_random_Pfannyw,
                schedule_coupled_weighted_random_PIu,
                schedule_coupled_weighted_random_PIuw,
                schedule_coupled_weighted_random_PIuf,
                schedule_coupled_weighted_random_PIufw,
                schedule_coupled_MCT_v1,
                schedule_coupled_MCT_v2,
                schedule_coupled_LW_v1,
                schedule_coupled_LW_v2,
                schedule_coupled_UD_v1,
                schedule_coupled_UD_v2,
                schedule_coupled_EMCT_v1,
                schedule_coupled_EMCT_v2,
                schedule_coupled_ELW_v1,
                schedule_coupled_ELW_v2,
                schedule_coupled_EUD_v1,
                schedule_coupled_EUD_v2,
                schedule_coupled_IE,
		schedule_coupled_IP,
		schedule_coupled_IR,
		schedule_coupled_IY,
		schedule_coupled_random_PP,
		schedule_coupled_weighted_random_Puu_PP,
		schedule_coupled_weighted_random_Puuw_PP,
		schedule_coupled_weighted_random_Pfanny_PP,
		schedule_coupled_weighted_random_Pfannyw_PP,
		schedule_coupled_weighted_random_PIu_PP,
		schedule_coupled_weighted_random_PIuw_PP,
		schedule_coupled_weighted_random_PIuf_PP,
		schedule_coupled_weighted_random_PIufw_PP,
		schedule_coupled_MCT_v1_PP,
		schedule_coupled_MCT_v2_PP,
		schedule_coupled_LW_v1_PP,
		schedule_coupled_LW_v2_PP,
		schedule_coupled_UD_v1_PP,
		schedule_coupled_UD_v2_PP,
		schedule_coupled_EMCT_v1_PP,
		schedule_coupled_EMCT_v2_PP,
		schedule_coupled_ELW_v1_PP,
		schedule_coupled_ELW_v2_PP,
		schedule_coupled_EUD_v1_PP,
		schedule_coupled_EUD_v2_PP,
		schedule_coupled_IE_PP,
		schedule_coupled_IP_PP,
		schedule_coupled_IR_PP,
		schedule_coupled_IY_PP,
		schedule_coupled_random_PE,
		schedule_coupled_weighted_random_Puu_PE,
		schedule_coupled_weighted_random_Puuw_PE,
		schedule_coupled_weighted_random_Pfanny_PE,
		schedule_coupled_weighted_random_Pfannyw_PE,
		schedule_coupled_weighted_random_PIu_PE,
		schedule_coupled_weighted_random_PIuw_PE,
		schedule_coupled_weighted_random_PIuf_PE,
		schedule_coupled_weighted_random_PIufw_PE,
		schedule_coupled_MCT_v1_PE,
		schedule_coupled_MCT_v2_PE,
		schedule_coupled_LW_v1_PE,
		schedule_coupled_LW_v2_PE,
		schedule_coupled_UD_v1_PE,
		schedule_coupled_UD_v2_PE,
		schedule_coupled_EMCT_v1_PE,
		schedule_coupled_EMCT_v2_PE,
		schedule_coupled_ELW_v1_PE,
		schedule_coupled_ELW_v2_PE,
		schedule_coupled_EUD_v1_PE,
		schedule_coupled_EUD_v2_PE,
		schedule_coupled_IE_PE,
		schedule_coupled_IP_PE,
		schedule_coupled_IR_PE,
		schedule_coupled_IY_PE,
		schedule_coupled_random_PR,
		schedule_coupled_weighted_random_Puu_PR,
		schedule_coupled_weighted_random_Puuw_PR,
		schedule_coupled_weighted_random_Pfanny_PR,
		schedule_coupled_weighted_random_Pfannyw_PR,
		schedule_coupled_weighted_random_PIu_PR,
		schedule_coupled_weighted_random_PIuw_PR,
		schedule_coupled_weighted_random_PIuf_PR,
		schedule_coupled_weighted_random_PIufw_PR,
		schedule_coupled_MCT_v1_PR,
		schedule_coupled_MCT_v2_PR,
		schedule_coupled_LW_v1_PR,
		schedule_coupled_LW_v2_PR,
		schedule_coupled_UD_v1_PR,
		schedule_coupled_UD_v2_PR,
		schedule_coupled_EMCT_v1_PR,
		schedule_coupled_EMCT_v2_PR,
		schedule_coupled_ELW_v1_PR,
		schedule_coupled_ELW_v2_PR,
		schedule_coupled_EUD_v1_PR,
		schedule_coupled_EUD_v2_PR,
		schedule_coupled_IE_PR,
		schedule_coupled_IP_PR,
		schedule_coupled_IR_PR,
		schedule_coupled_IY_PR,
		schedule_coupled_random_PY,
		schedule_coupled_weighted_random_Puu_PY,
		schedule_coupled_weighted_random_Puuw_PY,
		schedule_coupled_weighted_random_Pfanny_PY,
		schedule_coupled_weighted_random_Pfannyw_PY,
		schedule_coupled_weighted_random_PIu_PY,
		schedule_coupled_weighted_random_PIuw_PY,
		schedule_coupled_weighted_random_PIuf_PY,
		schedule_coupled_weighted_random_PIufw_PY,
		schedule_coupled_MCT_v1_PY,
		schedule_coupled_MCT_v2_PY,
		schedule_coupled_LW_v1_PY,
		schedule_coupled_LW_v2_PY,
		schedule_coupled_UD_v1_PY,
		schedule_coupled_UD_v2_PY,
		schedule_coupled_EMCT_v1_PY,
		schedule_coupled_EMCT_v2_PY,
		schedule_coupled_ELW_v1_PY,
		schedule_coupled_ELW_v2_PY,
		schedule_coupled_EUD_v1_PY,
		schedule_coupled_EUD_v2_PY,
		schedule_coupled_IE_PY,
		schedule_coupled_IP_PY,
		schedule_coupled_IR_PY,
		schedule_coupled_IY_PY,
};

extern struct global_t global;

/* Prototypes */
int run_simulation(algorithm_t alg);
void unassign_task(task_t *task);
void assign_task_to_host(task_t *task, host_t *host);
int num_tasks_assigned_to_host(host_t *host);
int num_tasks_not_yet_completed_on_host(host_t *host);
int num_tasks_not_yet_started_on_host(host_t *host);


config_t *new_config(void);
void copy_config(config_t *src, config_t *dst);
void free_config(config_t *config);

int pick_weighted(double *weights, int n);

#endif
